import ktc
import os
import onnx
from PIL import Image
import numpy as np

###  post process function  ###
import tensorflow as tf
import pathlib
import sys

val_map = [65, 970, 230, 809, 516, 57, 334, 415, 674, 332]

def softmax(A):
    e = np.exp(A)
    return e / np.sum(e, keepdims=True)


def postprocess(pre_output):
    #print(pre_output)
    score = softmax(pre_output)
    return score


###  pre process function  ###
def preprocess(pil_img):   
    image = tf.io.read_file(pil_img)
    image = tf.io.decode_jpeg(image, channels=3)
    image = tf.image.resize(image, [224, 224])
    image = tf.cast(image / 255., tf.float32)
    image = (image - 0.5) * 2;
    array = tf.Session().run(image)
    return array


# convert tflite model to onnx

bottom_nodes = []
bottom_nodes.append("resnet_model/dense/BiasAdd")
print("(1) convert tflite model to onnx ====>")
m = ktc.onnx_optimizer.tflite2onnx_flow("/data1/resnet50_v1/resnet50_v1.tflite", True, bottom_nodes)
m = ktc.onnx_optimizer.onnx2onnx_flow(m, eliminate_tail=True, opt_matmul=False)

onnx.save(m,'resnet50_tflite_opt.onnx')

# setup ktc config
print("(2) setup ktc config ====>")
km = ktc.ModelConfig(1000, "0001", "720", onnx_model=m)


# npu(only) performance simulation
print("(3) npu(only) performance simulation ====>")
eval_result = km.evaluate()
print("\nNpu performance evaluation result:\n" + str(eval_result))

## onnx model check
print("(4) onnx model check ====>")
in_data = preprocess('/data1/resnet50_v1/ILSVRC2012_img_val_10/ILSVRC2012_val_00000001.JPEG')
out_data = ktc.kneron_inference([in_data], onnx_file="/data1/resnet50_v1/resnet50_tflite_opt.onnx", input_names=["input_tensor"])
#det_res = postprocess(out_data)
#print(np.argmax(det_res))
print(np.argmax(out_data))


# load and normalize all image data from folder
img_name_list = []
img_list = []
img_cnt = 0
for (dir_path, _, file_names) in os.walk("/data1/resnet50_v1/ILSVRC2012_img_val_10"):
    for f_n in file_names:
        fullpath = os.path.join(dir_path, f_n)
        print("processing image: " + fullpath)
        img_cnt += 1
        img_name_list.append(f_n)
        img_data = preprocess(fullpath)
        img_list.append(img_data)

# fix point analysis
print("(5) fix point analysis ====>")
bie_model_path = km.analysis({"input_tensor": img_list})
print("\nFix point analysis done. Save bie model to '" + str(bie_model_path) + "'")

# bie model check
print("(6) bie model check ====>")
in_data = preprocess('/data1/resnet50_v1/ILSVRC2012_img_val_10/ILSVRC2012_val_00000001.JPEG')
radix = ktc.get_radix(img_list)
out_data = ktc.kneron_inference([in_data], bie_file=bie_model_path, input_names=["input_tensor"], radix=radix, platform=720)
#det_res = postprocess(out_data)
#print(np.argmax(det_res))
print(np.argmax(out_data))

# compile
print("(7) compile ====>")
nef_model_path = ktc.compile([km])
print("\nCompile done. Save Nef file to '" + str(nef_model_path) + "'")

# nef model check
img_name_list.sort()
Accruacy = 0
print("(8) nef model check ====>")
radix = ktc.get_radix(img_list)
for i in range(img_cnt):
    in_data = preprocess('/data1/resnet50_v1/ILSVRC2012_img_val_10/' + img_name_list[i])
    out_data = ktc.kneron_inference([in_data], nef_file=nef_model_path, radix=radix, platform=720)
    #det_res = postprocess(out_data)
    #print(det_res)
    val_result = np.argmax(out_data)
    if ((val_map[i] + 1) == val_result) :
       print(img_name_list[i], " : ", val_result, " == val_map(", val_map[i] + 1, ")")
       Accruacy += 1
    else :
       print(img_name_list[i], " : ", val_result, " != val_map(", val_map[i] + 1, ")")

print("Accuracy = ", Accruacy/img_cnt*100)

